package lel.flummi.skilloverlay.utils;

import lel.flummi.skilloverlay.api.PlayerProfile;
import lel.flummi.skilloverlay.api.PlayerSkills;
import lel.flummi.skilloverlay.api.records.Skills.Skill.Level;

import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.google.common.base.Splitter;

import net.minecraft.text.Text;

public class XPInformation {
	private static final XPInformation INSTANCE = new XPInformation();
	private final HashMap<String, SkillInfo> skillInfoMap = new HashMap<>();

	public static XPInformation getInstance() {
		return INSTANCE;
	}

	public static class SkillInfo {
		public int level;
		public float totalXp;
		public float currentXp;
		public float currentXpMax;
		public boolean fromApi = false;
	}

	public HashMap<String, Float> updateWithPercentage = new HashMap<>();

	public int correctionCounter = 0;

	private static final Splitter SPACE_SPLITTER = Splitter.on("  ").omitEmptyStrings().trimResults();
	private static final Pattern SKILL_PATTERN_PERCENTAGE =
		Pattern.compile("\\+(\\d+(?:,\\d+)*(?:\\.\\d+)?) (.+) \\((\\d\\d?(?:\\.\\d\\d?)?)%\\)");

	public HashMap<String, SkillInfo> getSkillInfoMap() {
		return skillInfoMap;
	}

	public SkillInfo getSkillInfo(String skillName) {
		return skillInfoMap.get(skillName.toLowerCase());
	}

	private String lastActionBar = null;

	public void onChatReceived(Text message, boolean overlay) {
		if (overlay) {
			String actionBar = StringUtils.cleanColour(message.getString());
			if (lastActionBar != null && lastActionBar.equalsIgnoreCase(actionBar)) {
				return;
			}
			lastActionBar = actionBar;

			List<String> components = SPACE_SPLITTER.splitToList(actionBar);

			for (String component : components) {
				Matcher matcher = SKILL_PATTERN_PERCENTAGE.matcher(component);
				if (matcher.matches()) {
					System.out.println(matcher);
					String skillS = matcher.group(2);
					String xpPercentageS = matcher.group(3).replace(",", "");

					float xpPercentage = Float.parseFloat(xpPercentageS);
					updateWithPercentage.put(skillS.toLowerCase(), xpPercentage);
				}
			}
		}
	}

	public void updateLevel(String skill, int level) {
		if (updateWithPercentage.containsKey(skill)) {
			SkillInfo skillInfo = new SkillInfo();
			skillInfo.totalXp = 0;
			skillInfo.level = level;

			List<Level> levelingArray = PlayerSkills.SKILLS.get(skill).levels();
			for (int i = 0; i < levelingArray.size(); i++) {
				float cap = levelingArray.get(i).level();
				if (i == level) {
					skillInfo.currentXp += updateWithPercentage.get(skill) / 100f * cap;
					skillInfo.totalXp += skillInfo.currentXp;
					skillInfo.currentXpMax = cap;
					break;
				} else {
					skillInfo.totalXp += cap;
				}
			}

			SkillInfo old = skillInfoMap.get(skill.toLowerCase());

			if (old.totalXp <= skillInfo.totalXp) {
				correctionCounter--;
				if (correctionCounter < 0) correctionCounter = 0;

				skillInfoMap.put(skill.toLowerCase(), skillInfo);
			} else if (++correctionCounter >= 10) {
				correctionCounter = 0;
				skillInfoMap.put(skill.toLowerCase(), skillInfo);
			}
		}
		updateWithPercentage.clear();
	}
}
